@@ -4,10 +4,10 @@ module Arel
       private
 
       # Locks are not supported in SQLite
-      def visit_Arel_Nodes_Lock o
+      def visit_Arel_Nodes_Lock o, a
       end
 
-      def visit_Arel_Nodes_SelectStatement o
+      def visit_Arel_Nodes_SelectStatement o, a
         o.limit = Arel::Nodes::Limit.new(-1) if o.offset && !o.limit
         super
       end
