@@ -7,16 +7,19 @@ module Arel
 
       private
 
-      DISPATCH = Hash.new do |hash, klass|
-        hash[klass] = "visit_#{(klass.name || '').gsub('::', '_')}"
+      DISPATCH = Hash.new do |hash, visitor_class|
+        hash[visitor_class] =
+          Hash.new do |method_hash, node_class|
+            method_hash[node_class] = "visit_#{(node_class.name || '').gsub('::', '_')}"
+          end
       end
 
       def dispatch
-        DISPATCH
+        DISPATCH[self.class]
       end
 
-      def visit object
-        send dispatch[object.class], object
+      def visit object, attribute = nil
+        send dispatch[object.class], object, attribute
       rescue NoMethodError => e
         raise e if respond_to?(dispatch[object.class], true)
         superklass = object.class.ancestors.find { |klass|
