@@ -1,58 +1,6 @@
 require 'helper'
 
 module Arel
-  class EngineProxy
-    attr_reader :executed
-    attr_reader :connection_pool
-    attr_reader :spec
-    attr_reader :config
-
-    def initialize engine
-      @engine = engine
-      @executed = []
-      @connection_pool = self
-      @spec = self
-      @config = { :adapter => 'sqlite3' }
-    end
-
-    def with_connection
-      yield self
-    end
-
-    def connection
-      self
-    end
-
-    def quote_table_name thing; @engine.connection.quote_table_name thing end
-    def quote_column_name thing; @engine.connection.quote_column_name thing end
-    def quote thing, column; @engine.connection.quote thing, column end
-    def columns table, message = nil
-      @engine.connection.columns table, message
-    end
-
-    def columns_hash
-      @engine.connection.columns_hash
-    end
-
-    def table_exists? name
-      @engine.connection.table_exists? name
-    end
-
-    def tables
-      @engine.connection.tables
-    end
-
-    def visitor
-      @engine.connection.visitor
-    end
-
-    def execute sql, name = nil, *args
-      @executed << sql
-    end
-    alias :update :execute
-    alias :delete :execute
-    alias :insert :execute
-  end
 
   describe 'select manager' do
     def test_join_sources
@@ -61,6 +9,13 @@ module Arel
       assert_equal "SELECT FROM 'foo'", manager.to_sql
     end
 
+    def test_manager_stores_bind_values
+      manager = Arel::SelectManager.new Table.engine
+      assert_equal [], manager.bind_values
+      manager.bind_values = [1]
+      assert_equal [1], manager.bind_values
+    end
+
     describe 'backwards compatibility' do
       describe 'project' do
         it 'accepts symbols as sql literals' do
@@ -435,20 +390,6 @@ module Arel
       end
     end
 
-    describe 'insert' do
-      it 'uses the select FROM' do
-        engine  = EngineProxy.new Table.engine
-        table   = Table.new :users
-        manager = Arel::SelectManager.new engine
-        manager.from table
-        manager.insert 'VALUES(NULL)'
-
-        engine.executed.last.must_be_like %{
-          INSERT INTO "users" VALUES(NULL)
-        }
-      end
-    end
-
     describe 'lock' do
       # This should fail on other databases
       it 'adds a lock node' do
@@ -888,9 +829,8 @@ module Arel
 
     describe 'delete' do
       it "copies from" do
-        engine  = EngineProxy.new Table.engine
         table   = Table.new :users
-        manager = Arel::SelectManager.new engine
+        manager = Arel::SelectManager.new Table.engine
         manager.from table
         stmt = manager.compile_delete
 
@@ -898,9 +838,8 @@ module Arel
       end
 
       it "copies where" do
-        engine  = EngineProxy.new Table.engine
         table   = Table.new :users
-        manager = Arel::SelectManager.new engine
+        manager = Arel::SelectManager.new Table.engine
         manager.from table
         manager.where table[:id].eq 10
         stmt = manager.compile_delete
@@ -930,12 +869,11 @@ module Arel
 
     describe 'update' do
       it 'copies limits' do
-        engine  = EngineProxy.new Table.engine
         table   = Table.new :users
-        manager = Arel::SelectManager.new engine
+        manager = Arel::SelectManager.new Table.engine
         manager.from table
         manager.take 1
-        stmt = manager.compile_update(SqlLiteral.new('foo = bar'))
+        stmt = manager.compile_update(SqlLiteral.new('foo = bar'), Arel::Attributes::Attribute.new(table, 'id'))
         stmt.key = table['id']
 
         stmt.to_sql.must_be_like %{
@@ -945,12 +883,11 @@ module Arel
       end
 
       it 'copies order' do
-        engine  = EngineProxy.new Table.engine
         table   = Table.new :users
-        manager = Arel::SelectManager.new engine
+        manager = Arel::SelectManager.new Table.engine
         manager.from table
         manager.order :foo
-        stmt = manager.compile_update(SqlLiteral.new('foo = bar'))
+        stmt = manager.compile_update(SqlLiteral.new('foo = bar'), Arel::Attributes::Attribute.new(table, 'id'))
         stmt.key = table['id']
 
         stmt.to_sql.must_be_like %{
@@ -960,22 +897,20 @@ module Arel
       end
 
       it 'takes a string' do
-        engine  = EngineProxy.new Table.engine
         table   = Table.new :users
-        manager = Arel::SelectManager.new engine
+        manager = Arel::SelectManager.new Table.engine
         manager.from table
-        stmt = manager.compile_update(SqlLiteral.new('foo = bar'))
+        stmt = manager.compile_update(SqlLiteral.new('foo = bar'), Arel::Attributes::Attribute.new(table, 'id'))
 
         stmt.to_sql.must_be_like %{ UPDATE "users" SET foo = bar }
       end
 
       it 'copies where clauses' do
-        engine  = EngineProxy.new Table.engine
         table   = Table.new :users
-        manager = Arel::SelectManager.new engine
+        manager = Arel::SelectManager.new Table.engine
         manager.where table[:id].eq 10
         manager.from table
-        stmt = manager.compile_update(table[:id] => 1)
+        stmt = manager.compile_update({table[:id] => 1}, Arel::Attributes::Attribute.new(table, 'id'))
 
         stmt.to_sql.must_be_like %{
           UPDATE "users" SET "id" = 1 WHERE "users"."id" = 10
@@ -983,13 +918,12 @@ module Arel
       end
 
       it 'copies where clauses when nesting is triggered' do
-        engine  = EngineProxy.new Table.engine
         table   = Table.new :users
-        manager = Arel::SelectManager.new engine
+        manager = Arel::SelectManager.new Table.engine
         manager.where table[:foo].eq 10
         manager.take 42
         manager.from table
-        stmt = manager.compile_update(table[:id] => 1)
+        stmt = manager.compile_update({table[:id] => 1}, Arel::Attributes::Attribute.new(table, 'id'))
 
         stmt.to_sql.must_be_like %{
           UPDATE "users" SET "id" = 1 WHERE "users"."id" IN (SELECT "users"."id" FROM "users" WHERE "users"."foo" = 10 LIMIT 42)
@@ -997,11 +931,10 @@ module Arel
       end
 
       it 'executes an update statement' do
-        engine  = EngineProxy.new Table.engine
         table   = Table.new :users
-        manager = Arel::SelectManager.new engine
+        manager = Arel::SelectManager.new Table.engine
         manager.from table
-        stmt = manager.compile_update(table[:id] => 1)
+        stmt = manager.compile_update({table[:id] => 1}, Arel::Attributes::Attribute.new(table, 'id'))
 
         stmt.to_sql.must_be_like %{
           UPDATE "users" SET "id" = 1
@@ -1019,16 +952,14 @@ module Arel
 
       it 'takes strings' do
         manager = Arel::SelectManager.new Table.engine
-        manager.project Nodes::SqlLiteral.new('*')
+        manager.project '*'
         manager.to_sql.must_be_like %{ SELECT * }
       end
 
       it "takes sql literals" do
         manager = Arel::SelectManager.new Table.engine
         manager.project Nodes::SqlLiteral.new '*'
-        manager.to_sql.must_be_like %{
-          SELECT *
-        }
+        manager.to_sql.must_be_like %{ SELECT * }
       end
     end
 
