@@ -18,7 +18,7 @@ module Arel
       it 'can define a dispatch method' do
         visited = false
         viz = Class.new(Arel::Visitors::Visitor) {
-          define_method(:hello) do |node|
+          define_method(:hello) do |node, attribute|
             visited = true
           end
 
@@ -92,7 +92,14 @@ module Arel
         assert_equal 'omg(*, *)', @visitor.accept(function)
       end
 
-      describe 'equality' do
+      describe 'Nodes::Equality' do
+        it "should escape strings" do
+          test = Table.new(:users)[:name].eq 'Aaron Patterson'
+          @visitor.accept(test).must_be_like %{
+            "users"."name" = 'Aaron Patterson'
+          }
+        end
+
         it 'should handle false' do
           sql = @visitor.accept Nodes::Equality.new(false, false)
           sql.must_be_like %{ 'f' = 'f' }
@@ -103,6 +110,23 @@ module Arel
           sql = @visitor.accept Nodes::Equality.new(table[:id], '1-fooo')
           sql.must_be_like %{ "users"."id" = 1 }
         end
+
+        it 'should handle nil' do
+          sql = @visitor.accept Nodes::Equality.new(@table[:name], nil)
+          sql.must_be_like %{ "users"."name" IS NULL }
+        end
+      end
+
+      describe 'Nodes::NotEqual' do
+        it 'should handle false' do
+          sql = @visitor.accept Nodes::NotEqual.new(@table[:active], false)
+          sql.must_be_like %{ "users"."active" != 'f' }
+        end
+
+        it 'should handle nil' do
+          sql = @visitor.accept Nodes::NotEqual.new(@table[:name], nil)
+          sql.must_be_like %{ "users"."name" IS NOT NULL }
+        end
       end
 
       it "should visit string subclass" do
@@ -194,6 +218,40 @@ module Arel
         @visitor.accept(test).must_be_like %{ "users"."bool" = 't' }
       end
 
+      describe "Nodes::Matches" do
+        it "should know how to visit" do
+          node = @table[:name].matches('foo%')
+          @visitor.accept(node).must_be_like %{
+            "users"."name" LIKE 'foo%'
+          }
+        end
+
+        it 'can handle subqueries' do
+          subquery = @table.project(:id).where(@table[:name].matches('foo%'))
+          node = @attr.in subquery
+          @visitor.accept(node).must_be_like %{
+            "users"."id" IN (SELECT id FROM "users" WHERE "users"."name" LIKE 'foo%')
+          }
+        end
+      end
+
+      describe "Nodes::DoesNotMatch" do
+        it "should know how to visit" do
+          node = @table[:name].does_not_match('foo%')
+          @visitor.accept(node).must_be_like %{
+            "users"."name" NOT LIKE 'foo%'
+          }
+        end
+
+        it 'can handle subqueries' do
+          subquery = @table.project(:id).where(@table[:name].does_not_match('foo%'))
+          node = @attr.in subquery
+          @visitor.accept(node).must_be_like %{
+            "users"."id" IN (SELECT id FROM "users" WHERE "users"."name" NOT LIKE 'foo%')
+          }
+        end
+      end
+
       describe "Nodes::Ordering" do
         it "should know how to visit" do
           node = @attr.desc
@@ -378,15 +436,6 @@ module Arel
         end
       end
 
-      describe 'Equality' do
-        it "should escape strings" do
-          test = Table.new(:users)[:name].eq 'Aaron Patterson'
-          @visitor.accept(test).must_be_like %{
-            "users"."name" = 'Aaron Patterson'
-          }
-        end
-      end
-
       describe 'Constants' do
         it "should handle true" do
           test = Table.new(:users).create_true
